@@ -27,7 +27,7 @@
 
 #include "config.h"
 #include "gnome-vfs-application-registry.h"
-
+#include "gnome-vfs-configuration.h"
 #include "gnome-vfs-mime-handlers.h"
 #include "gnome-vfs-mime-private.h"
 #include "gnome-vfs-mime.h"
@@ -46,6 +46,11 @@
 #include <time.h>
 #include <unistd.h>
 
+/* Defined there and not with the other constants in the header file not to 
+ * break 2.0 api freeze
+ */
+#define GNOME_VFS_APPLICATION_REGISTRY_USES_GNOMEVFS "uses_gnomevfs"
+
 #if !defined getc_unlocked && !defined HAVE_GETC_UNLOCKED
 # define getc_unlocked(fp) getc (fp)
 #endif
@@ -181,6 +186,8 @@ application_lookup_or_create (const char *app_id, gboolean user_owned)
 
 	g_return_val_if_fail(app_id != NULL, NULL);
 
+	gnome_vfs_application_registry_init ();
+
 	application = g_hash_table_lookup (global_applications, app_id);
 	if (application != NULL) {
 		if ( ! user_owned) {
@@ -703,7 +710,7 @@ load_application_info_from (const char *filename, gboolean user_owned)
 	Application *application;
 	char *key;
 	char *lang;
-	
+
 	fp = fopen (filename, "r");
 	if (fp == NULL)
 		return;
@@ -969,12 +976,12 @@ gnome_vfs_application_registry_init (void)
 			  user_registry_dir.dirname);
 	}
 
-	/*
-	 * Load
+	/* Things have been initialized flag it as ready so that we can load
+	 * the applications without attempting to reinitialize
 	 */
-	load_application_info ();
-
 	gnome_vfs_application_registry_initialized = TRUE;
+
+	load_application_info ();
 }
 
 /*
@@ -1592,7 +1599,8 @@ gnome_vfs_application_registry_supports_uri_scheme (const char *app_id,
 						    const char *uri_scheme)
 {
 	Application *application;
-
+	gboolean uses_gnomevfs;
+	
 	g_return_val_if_fail (app_id != NULL, FALSE);
 	g_return_val_if_fail (uri_scheme != NULL, FALSE);
 
@@ -1602,7 +1610,10 @@ gnome_vfs_application_registry_supports_uri_scheme (const char *app_id,
 	if (application == NULL)
 		return FALSE;
 
+	uses_gnomevfs = real_get_bool_value (application, GNOME_VFS_APPLICATION_REGISTRY_USES_GNOMEVFS, NULL);
+
 	if (strcmp (uri_scheme, "file") == 0 &&
+	    uses_gnomevfs == FALSE &&
 	    application->supported_uri_schemes == NULL &&
 	    application->user_application->supported_uri_schemes == NULL) {
 		return TRUE;
@@ -1622,6 +1633,21 @@ gnome_vfs_application_registry_supports_uri_scheme (const char *app_id,
 				 (GCompareFunc) strcmp) != NULL)) {
 		return TRUE;
 	}
+	/* check in the list of uris supported by gnome-vfs if necessary */
+	else if (uses_gnomevfs) {
+		GList *supported_uris;
+		gboolean res;
+		
+		supported_uris = gnome_vfs_configuration_get_methods_list();
+		res = (g_list_find_custom(supported_uris,
+					  /*glib is const incorrect*/(gpointer) uri_scheme,
+					  (GCompareFunc) strcmp) != NULL);
+
+		g_list_foreach(supported_uris, (GFunc) g_free, NULL);
+		g_list_free(supported_uris);
+
+		return res;
+	}
 
 	return FALSE;
 }
@@ -1849,6 +1875,7 @@ gnome_vfs_application_registry_get_mime_application (const char *app_id)
 {
 	Application *i_application;
 	GnomeVFSMimeApplication *application;
+	gboolean uses_gnomevfs = FALSE;
 
 	g_return_val_if_fail (app_id != NULL, NULL);
 
@@ -1887,6 +1914,29 @@ gnome_vfs_application_registry_get_mime_application (const char *app_id)
 			 GNOME_VFS_APPLICATION_REGISTRY_REQUIRES_TERMINAL,
 			 NULL);
 
+	uses_gnomevfs = real_get_bool_value (i_application, GNOME_VFS_APPLICATION_REGISTRY_USES_GNOMEVFS, NULL);
+
+	if (uses_gnomevfs) {
+		GList *methods_list = 
+			gnome_vfs_configuration_get_methods_list();
+		GList *l;
+		if (application->expects_uris == GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS) {
+			application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS;
+		}
+		for (l = methods_list; l != NULL; l = l->next) {
+			if (g_list_find_custom (application->supported_uri_schemes,
+						/*glib is const incorrect*/(gpointer) l->data,
+						(GCompareFunc) strcmp) == NULL) {
+				application->supported_uri_schemes = 
+					g_list_prepend(application->supported_uri_schemes, 
+						       l->data);
+			} else {
+				g_free(l->data);
+			}			
+		}
+		g_list_free(methods_list);
+	}	
+
 	return application;
 }
 
